@@ -200,6 +200,15 @@ class Worker(Monque):
 
 
     def main_loop(self):
+        try:
+            return self.main_loop_protected()
+        except:
+            ex = sys.exc_info()
+            sys.stderr.write("%s: Uncaught exception in main_loop:%s\n%s\n" %
+                             (self.worker_name,str(ex[1]),traceback.format_exc(ex[2])))
+            self.stop_running()
+
+    def main_loop_protected(self):
         """
         Main loop for the worker, runs in the 'main' thread.
         Just waits indefinitely while the task thread consumes tasks,
@@ -221,7 +230,7 @@ class Worker(Monque):
 
             # After waking up, check if timeout conditions are reached
             if self.check_timeout():
-                self.logger.warning("%s: run() finishing on run limit" % (self.worker_name))
+                self.logger.info("%s: run() finishing on run limit" % (self.worker_name))
                 self.stop_running()
 
             # Periodically update the "current workers" record
@@ -243,8 +252,8 @@ class Worker(Monque):
 
         if self.max_run_count and \
                 self.run_count >= self.max_run_count:
-            self.logger.warning("%s: worker reached max_run_count (%d)" %
-                                (self.worker_name,self.run_count))
+            self.logger.info("%s: worker reached max_run_count (%d)" %
+                             (self.worker_name,self.run_count))
             return True
 
         if self.max_exception_count and \
@@ -256,8 +265,8 @@ class Worker(Monque):
         if self.max_run_time and current:
             # use total run time, not current
             if self.run_time >= self.max_run_time:
-                self.logger.warning("%s: worker reached max_run_time (%s)" %
-                                    (self.worker_name,self.run_time))
+                self.logger.info("%s: worker reached max_run_time (%s)" %
+                                 (self.worker_name,self.run_time))
                 return True
 
         if self.max_idle_time and not current:
@@ -270,8 +279,8 @@ class Worker(Monque):
                              (idle_time.seconds + idle_time.days * 24 * 3600) * 10**6) / 10**6
 
             if idle_secs >= self.max_idle_time:
-                self.logger.warning("%s: worker reached max_idle_time (%s)" %
-                                    (self.worker_name,idle_secs))
+                self.logger.info("%s: worker reached max_idle_time (%s)" %
+                                 (self.worker_name,idle_secs))
                 return True
 
         return False
@@ -320,6 +329,15 @@ class Worker(Monque):
 
 
     def task_loop(self):
+        try:
+            return self.task_loop_protected()
+        except:
+            ex = sys.exc_info()
+            sys.stderr.write("%s: Uncaught exception in task_loop:%s\n%s\n" %
+                             (self.worker_name,str(ex[1]),traceback.format_exc(ex[2])))
+            self.stop_running()
+
+    def task_loop_protected(self):
         """
         Loop in which the worker waits for a task to be available, then executes it.
         Multiple threads may be running the same loop.
@@ -414,13 +432,13 @@ class Worker(Monque):
             return self.known_tasks[task_name]
 
         task_class = Task.find_task_class(class_name)
-        task = self.known_tasks[task_name] = task_class(monque=self)
+        task = self.known_tasks[task_name] = task_class(monque=self,logger=self.logger)
         return task
 
 
     def can_run_task(self,posted_task):
         """
-        Check task constraints prior to running tgis task instance
+        Check task constraints prior to running this task instance
         """
         # TODO: not implemented...
         return True
@@ -434,12 +452,13 @@ class Worker(Monque):
         with self.lock:
             self.current_task = (posted_task,datetime.datetime.utcnow())
             self.idle_since = None
-                                                
+            counter = self.run_count
+
         task = posted_task.task
         args = posted_task.args
         kwargs = posted_task.kwargs
         self.logger.info("%s: run[%d]: %s args=%s kwargs=%s" %
-                         (self.worker_name,self.run_count,
+                         (self.worker_name,counter,
                           posted_task.name,args,kwargs))
 
         started = time.time()
@@ -449,7 +468,12 @@ class Worker(Monque):
             self.store_task_result(posted_task,result)
         except:
             ended = time.time()
-            self.store_task_exception(posted_task,sys.exc_info())
+            ex = sys.exc_info()
+            self.logger.warning("%s: run[%d]: Exception in %s: %s\n%s" %
+                                (self.worker_name,counter,
+                                 posted_task.name,
+                                 ex[1],traceback.format_exc(ex[2])))
+            self.store_task_exception(posted_task,ex)
 
             with self.lock:
                 self.exception_count += 1
@@ -508,6 +532,15 @@ class Worker(Monque):
 
 
     def activity_loop(self):
+        try:
+            return self.activity_loop_protected()
+        except:
+            ex = sys.exc_info()
+            sys.stderr.write("%s: Uncaught exception in activity_loop:%s\n%s\n" %
+                             (self.worker_name,str(ex[1]),traceback.format_exc(ex[2])))
+            self.stop_running()
+
+    def activity_loop_protected(self):
         """
         In order to keep from activity polling the task queue (tasks collection),
         the activity_log is used. activity_log is a capped collection, which allows a tailable
@@ -580,6 +613,15 @@ class Worker(Monque):
 
 
     def control_loop(self):
+        try:
+            return self.control_loop_protected()
+        except:
+            ex = sys.exc_info()
+            sys.stderr.write("%s: Uncaught exception in control_loop:%s\n%s\n" %
+                             (self.worker_name,str(ex[1]),traceback.format_exc(ex[2])))
+            self.stop_running()
+
+    def control_loop_protected(self):
         """
         Wait for new messages on the control channel (pause, etc)
         """
@@ -617,10 +659,15 @@ class Worker(Monque):
 
                     self.logger.info("%s: control msg = %s" % (self.worker_name,msg))
 
-                    try:
-                        self.handle_control_msg(msg)
-                    except:
-                        pass
+                    with self.lock:
+                        try:
+                            self.handle_control_msg(msg)
+                        except:
+                            ex = sys.exc_info()
+                            self.logger.warning("%s: exception handling control msg: msg = %s\n%s\n%s" %
+                                                (self.worker_name,msg,
+                                                 str(ex[1]),traceback.format_exc(ex[2])))
+
             except pymongo.errors.OperationFailure, ex:
                 self.logger.error("%s: failed in control_loop due to: %s" %
                                   (self.worker_name,ex))
@@ -653,21 +700,21 @@ class Worker(Monque):
     def pause(self):
         if not self.paused:
             self.paused = True
-            self.logger.warning("%s: PAUSED -- issue the 'resume' command to continue" % (self.worker_name))
+            self.logger.info("%s: PAUSED -- issue the 'resume' command to continue" % (self.worker_name))
             with self.lock:
                 self.lock.notifyAll()
 
     def resume(self):
         if self.paused:
             self.paused = False
-            self.logger.warning("%s: RESUMED" % (self.worker_name))
+            self.logger.info("%s: RESUMED" % (self.worker_name))
             with self.lock:
                 self.lock.notifyAll()
 
     def stop(self):
         if self.running:
-            self.logger.warning("%s: STOPPED -- issue the 'resume' command before starting workers" %
-                                (self.worker_name))
+            self.logger.info("%s: STOPPED -- issue the 'resume' command before starting workers" %
+                             (self.worker_name))
 
             self.stop_running()
 
